subroutine CalcEnergy(Pos, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1., pi = 2 * ACOS(0.0)
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12, z9, z3, iz3, iz9, zij
    real(8) :: rc2, Shift, rij_s
    integer :: i, j
    !Compute the Lennard-Jones shift value before the pairwise loop
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
    PEnergy = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        zij = Pos(i,2)
        z3 = zij * zij * zij
        iz3 = 1. / z3
        iz9 = iz3 * iz3 * iz3
        PEnergy = PEnergy + 8. * pi * 1. * 0.74 * (iz9 - iz3)
        do j = i + 1, NAtom - 1
            if (j == i + 1 .and. mod(j, NAtom) > 0) then
                !this is a bonded interaction;
        		!use the harmonic bond potential here
                rij = Pos(j,:) - Posi
                !rij = rij - L * dnint(rij / L)
                rij_s = sqrt(sum(rij * rij))
        		PEnergy = PEnergy + (k * 0.5) * (rij_s-r0) ** 2.
            else
                !this is a nonbonded interaction;
            	!use the Lennard-Jones potential here
            	rij = Pos(j,:) - Posi
            	!rij = rij - L * dnint(rij / L)
            	!Compute only the squared distance and compare to the square cut                
            	d2 = sum(rij * rij)
            	if (d2 > rc2) then
            	    cycle
            	end if
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                !Calculate the PEnergy, U
            	PEnergy = PEnergy + 4. * (id12 - id6) + Shift
        	end if
    	enddo
    enddo   

end subroutine

subroutine CalcEnergyForces(Pos, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 3000., r0 = 1., pi = 2 * ACOS(0.0)
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d2, id2, id6, id12, z2, z3, iz3, iz9, zij
    real(8) :: rc2, Shift, rij_s
    integer :: i, j
    !Compute the Lennard-Jones shift value before the pairwise loop
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
    PEnergy = 0.
    Forces = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        zij = Pos(i,2)
        z2 = zij * zij
        z3 = zij * zij * zij
        iz3 = 1. / z3
        iz9 = iz3 * iz3 * iz3
        PEnergy = PEnergy + 8. * pi * 1. * 0.74 * (iz9 - iz3)
        Forces(i,2) = Forces(i,2) + 8. * pi * 1. * 0.74 * (-9. * iz9 + 3. * iz3) * iz3 * z2
        do j = i + 1, NAtom - 1
            if (j == i + 1 .and. mod(j, NAtom) > 0) then
            	!this is a bonded interaction;
        		!use the harmonic bond potential here
        		rij = Pos(j,:) - Posi
        		!rij = rij - L * dnint(rij / L)
        		rij_s = sqrt(sum(rij * rij))
        		PEnergy = PEnergy + (k * 0.5) * (rij_s-r0) ** 2.
        		Fij = k * (rij_s - r0) * rij / rij_s
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
    	    else
        		!this is a nonbonded interaction;
        		!use the Lennard-Jones potential here
    	        rij = Pos(j,:) - Posi
            	!rij = rij - L * dnint(rij / L)
                !Compute only the squared distance and compare to the square cut                
        		d2 = sum(rij * rij)
        		if (d2 > rc2) then
        		    cycle
        		end if
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                !Calculate the PEnergy, U, and Force from W,the virial
    	        PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                Fij = rij * ((-48. * id12 + 24. * id6) * id2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij    
    	    end if
    	enddo
    	
    enddo
end subroutine

subroutine VVIntegrate(Pos, Vel, Accel, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in) :: rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Accel(0,:) = 0.
    Vel(0,:) = 0.
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    Accel(0,:) = 0.
    Vel(0,:) = 0.
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine